<?php
namespace Teknisa\PLA\Modules\Mymenu\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

class Mym00110InformativoController extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $dataSourceName = 'informativomymenu_filial_localpesq';
    protected $dataSourceNameToSave = 'informativomymenu';
    
    protected $fileServerFilesUrl;
    public function __construct($crudService, $parameterBag, $fileServerFilesUrl){
        parent::__construct($crudService, $parameterBag);
        $this->fileServerFilesUrl = $fileServerFilesUrl;
    }
    
    public function save(DTO\Request $request, DTO\Response $response){
		try {
			$dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
			$rowsToSave = [];
			foreach($rows as $row) {
			    $imageCol = array_key_exists("URLIMAGEM", $row) ? $row['URLIMAGEM'] : false;
			    if ($imageCol && gettype($imageCol) !== "string") {
			        $imageColValue = null;
    			    if (gettype($imageCol) === "array") {
    			        if (sizeof($imageCol) > 0) {
    			            $fileObject = $imageCol[0];
    			            if (gettype($fileObject) === "array" && array_key_exists("type", $fileObject) && array_key_exists("path", $fileObject)) {
        			            $fileType = $fileObject['type'];
        			            if (substr($fileType, 0, 5) === "image") {
        			                $imageColValue = $fileObject['path'];
        			            }
    			            }
    			        }
    			    }
    			    $row['URLIMAGEM'] = $imageColValue;
			    } else if ($imageCol && gettype($imageCol) === "string") {
			       $row['URLIMAGEM'] = str_replace($this->fileServerFilesUrl, "", $row['URLIMAGEM']);
			    }
			    array_push($rowsToSave, $row);
			}
			$this->crudService->save($dataSourceName, $rowsToSave);
            $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
    }

	public function find(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $filterCriteria->setDataSourceName($this->dataSourceName);
            $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
            $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, $this->parameterBagColumns);
            $this->crudService->find($response, $newFilterCriteria);
	        
	        $responseDataSourceName = substr($request->getRoutePath(), 1);
	        $responseRows = [];
	        foreach ($response->getDataSets()[0]->getRows() as $row) {
	            $imageCol = array_key_exists("URLIMAGEM", $row) ? $row['URLIMAGEM'] : false;
			    if ($imageCol && gettype($imageCol) === "string") {
    			    $row['URLIMAGEM'] = $this->fileServerFilesUrl.$row['URLIMAGEM'];
			    }
	            array_push($responseRows, $row);
	        }
	        
	        $responseDataSet = new DataSet($responseDataSourceName, $responseRows);
	        $response->setDataSets([$responseDataSet]);
	        
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}