<?php
namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class Pla02900FrequenciaOpcao extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $parameterBag;
    protected $util;
    protected $dataSourceManager;
    protected $connection;
    protected $RestrContrService;

    protected $errors = array();

    public function __construct($crudService, $parameterBag, $util, $dataSourceManager, $connection, $RestrContrService) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->util = $util;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection = $connection;
        $this->RestrContrService = $RestrContrService;
    }

    //Cascade save Vigencia->Servico->Grupo
    public function saveVigencia($vigencia, $response, $dataSourceName = "vigfreqopcagr") {
        try {
            $vigencia["NRVIGENCIA"] = $this->RestrContrService->getNewCode($dataSourceName);
            $dataSet = new DataSet($dataSourceName, array($vigencia));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrvigencia = $rowsToSave[0]["NRVIGENCIA"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);
            return $nrvigencia;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveServico($servico, $response) {
        try {
            $dataSourceName = "freqopcagr";
            $servico["NRFREQOPCAGR"] = $this->RestrContrService->getNewCode($dataSourceName);
            $dataSet = new DataSet($dataSourceName, array($servico));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrfreqopcagr = $rowsToSave[0]["NRFREQOPCAGR"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);
            return $nrfreqopcagr;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveGrupo(DTO\Request $request, DTO\Response $response) {
        try {
            
            $row = $request->getRow();
            $this->connection->beginTransaction();
            if (isset($row["vigencia"])) {
                $nrvigencia = $this->saveVigencia($row["vigencia"], $response);
            }
            if (isset($row["servico"])) {
                $row["servico"]["NRVIGENCIA"] = isset($nrvigencia) ? $nrvigencia : $row["servico"]["NRVIGENCIA"];
                $nrfreqopcagr = $this->saveServico($row["servico"], $response);
            }
            $row["grupo"]["NRFREQOPCAGR"] = isset($nrfreqopcagr) ? $nrfreqopcagr : $row["grupo"]["NRFREQOPCAGR"];
            $this->util->validateDuplicatedRecords($row["grupo"], "itfreqopcagr", array("NRGRUPOPRATOSERV", "NRFREQOPCAGR"));
            $this->dataSourceName = "itfreqopcagr";
            $row["grupo"]["NRITFREQOPCAGR"] = $this->RestrContrService->getNewCode($this->dataSourceName);
            $dataSet = new DataSet($this->dataSourceName, array($row["grupo"]));
            $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
            parent::save($newRequest, $response);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    //Importação - Exportação
    public function servicosImportExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();

            $servicos = $row["NMSERVICO"];
            if (isset($row["vigencia"])) {
                $nrvigenciadest = $this->saveVigencia($row["vigencia"], $response);
            } else {
                $nrvigenciadest =  $row["NRVIGENCIA"];
            }
            $tpoper = $row["TPOPER"];

            if ($tpoper == "S") {
                $this->deleteFreqopcagrByNrvigencia($nrvigenciadest);
            }
            foreach ($servicos as $servico) {
                $grupos = $this->getItfreqopcagr($servico["NRFREQOPCAGR"]);
                $nrfreqopcagrdest = $this->saveServicoImportExport($servico, $row, $nrvigenciadest);
                $gruposExisting = $this->getItfreqopcagr($nrfreqopcagrdest);
                $gruposExisting = array_map(function($grupo){
                    return $grupo["NRGRUPOPRATOSERV"];
                }, $gruposExisting);
                $gruposToSave = array();
                foreach ($grupos as &$grupo) {
                    if (in_array($grupo["NRGRUPOPRATOSERV"], $gruposExisting)) {
                        continue;
                    } else {
                        $grupo["NRFREQOPCAGR"] = $nrfreqopcagrdest;
                        $grupo["NRITFREQOPCAGR"] = $this->RestrContrService->getNewCode("ITFREQOPCAGR");
                        $grupo["__is_new"] = true;
                        array_push($gruposToSave, $grupo);
                    }
                }
                $dataSourceName = "itfreqopcagr";
                $dataSet = new DataSet($dataSourceName, $gruposToSave);
                $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
                $dataSet = new DataSet($dataSourceName, $rowsToSave);
                $this->dataSourceManager->persist($dataSet);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function saveServicoImportExport($servico, $vigencia, $nrvigenciadest) {
        $dataSourceName = "freqopcagr";

        $conditions = $this->RestrContrService->buildConditions(array("CDSERVFILI", "NRVIGENCIA"), array($servico["CDSERVFILI"], $nrvigenciadest));
        $freqopcagr = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        if (!empty($freqopcagr)) {
            $nrfreqopcagr =  $freqopcagr[0]["NRFREQOPCAGR"];
        } else {
            $servico["NRVIGENCIA"] = $nrvigenciadest;
            $servico["DTVIGENCIA"] = $vigencia["DTVIGENCIA"];
            $servico["CDFILIAL"] = $vigencia["CDFILIAL"];
            $servico["DSFREQOPCAGR"] = $vigencia["DSFREQOPCAGR"];
            $nrfreqopcagr = $this->RestrContrService->getNewCode("FREQOPCAGR");
            $servico["NRFREQOPCAGR"] = $nrfreqopcagr;
            $servico["__is_new"] = true;
            
            $dataSet = new DataSet($dataSourceName, array($servico));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
        }
        return $nrfreqopcagr;
    }

    public function getItfreqopcagr($nrfreqopcagr) {
        $conditions = $this->RestrContrService->buildConditions(array("NRFREQOPCAGR"), array($nrfreqopcagr));
        return $this->crudService->getFilteredDataSet("itfreqopcagr", $conditions)->getRows();
    }

    private function deleteFreqopcagrByNrvigencia($nrvigencia) {
        $sql = "DELETE FROM FREQOPCAGR WHERE NRVIGENCIA = :NRVIGENCIA AND NRORG = :NRORG";
        $params = array(
            "NRVIGENCIA"  => $nrvigencia,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    public function vigenciaExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();
            $vigenciaOrig = $row["VIGENCIAORIG"];
            $tables = $row["VIGENCIADEST"];
            foreach($tables as $table) {
                if (in_array(strtoupper($table), $this->RestrContrService->getTables())) { // Prevent SQL injection
                    $row = array(
                        "CDFILIAL" => $vigenciaOrig["CDFILIAL"],
                        "DTVIGENCIA" => $vigenciaOrig["DTVIGENCIA"],
                        "DTVIGENCIAFIM" => $vigenciaOrig["DTVIGENCIAFIM"],
                        "DESCRICAO" => $vigenciaOrig["DESCRICAO"],
                        "IDTIPORESTRICAO" => $vigenciaOrig["IDTIPORESTRICAO"],
                        "CDDESCOBSFILIAL" => $vigenciaOrig["CDDESCOBSFILIAL"],
                        "dataSourceName" => strtolower($table),
                        "fieldsToValidate" => array("CDFILIAL", "DTVIGENCIA"),
                        "__is_new" => true
                    );
                    try {
                        $this->RestrContrService->validateRegisterVigencia($row);
                        if (isset($row["DTVIGENCIAFIM"]) && strlen($row["DTVIGENCIAFIM"])) {
                            $this->RestrContrService->validateIntervalOverlap(null, $row["DTVIGENCIA"], $row["DTVIGENCIAFIM"], $row["dataSourceName"], $row["CDFILIAL"], $row["IDTIPORESTRICAO"]);
                        }
                        $this->saveVigencia($row, $response, $row["dataSourceName"]);
                    } catch( \Exception $e) {
                        if ($e->getMessage() == "registerExists") {
                            array_push($this->errors, $table);
                        } else {
                            throw $e;
                        }
                    }
                }
            }
            
            $this->connection->commit();
            if (!empty($this->errors)) {
                $response->addDataSet(new DataSet("errors", $this->errors));
                $response->setError(new Error("vigenciaExportErrors", 500));
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}
